@@ -1,4 +1,4 @@
-/* $Id: VBoxNetFltInternal.h 157380 2023-05-12 13:25:17Z bird $ */
+/* $Id: VBoxNetFltInternal.h 163807 2024-07-05 16:06:50Z aleksey $ */
 /** @file
  * VBoxNetFlt - Network Filter Driver (Host), Internal Header.
  */
@@ -202,6 +202,8 @@ typedef struct VBOXNETFLTINS
             struct sk_buff_head   XmitQueue;
             struct work_struct    XmitTask;
 #  endif
+            /** Unique identifier of network namespace of device to attach to. */
+            uint32_t uNamespaceInode;
             /** @} */
 # elif defined(RT_OS_SOLARIS)
             /** @name Solaris instance data.
