@@ -1,4 +1,4 @@
-/* $Id: strformatrt.cpp 155244 2023-01-17 14:15:46Z bird $ */
+/* $Id: strformatrt.cpp 156956 2023-04-19 21:42:39Z bird $ */
 /** @file
  * IPRT - IPRT String Formatter Extensions.
  */
@@ -686,7 +686,8 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                                     cch = pfnOutput(pvArgOutput, RT_STR_TUPLE(" - "));
                                 else
                                     cch = pfnOutput(pvArgOutput, RT_STR_TUPLE(": "));
-                                cch += pfnOutput(pvArgOutput, u.pErrInfo->pszMsg, u.pErrInfo->cbMsg);
+                                cch += pfnOutput(pvArgOutput, u.pErrInfo->pszMsg,
+                                                 RTStrNLen(u.pErrInfo->pszMsg, u.pErrInfo->cbMsg));
                             }
                             return cch;
                         }
