@@ -1,10 +1,10 @@
-/* $Id: magics.h 153224 2022-08-22 17:43:14Z klaus $ */
+/* $Id: magics.h 155541 2023-01-26 15:59:04Z aeichner $ */
 /** @file
  * IPRT - Internal header defining The Magic Numbers.
  */
 
 /*
- * Copyright (C) 2007-2022 Oracle and/or its affiliates.
+ * Copyright (C) 2007-2023 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -237,14 +237,6 @@
 #define RTTCPSERVER_MAGIC               UINT32_C(0x19470304)
 /** Magic value for RTTCPSERVER::u32Magic. (Harlan Ellison) */
 #define RTUDPSERVER_MAGIC               UINT32_C(0x19340527)
-/** The value of RTTAR::u32Magic. (Donald Ervin Knuth) */
-#define RTTAR_MAGIC                     UINT32_C(0x19380110)
-/** The value of RTTAR::u32Magic after RTTarClose(). */
-#define RTTAR_MAGIC_DEAD                ~RTTAR_MAGIC
-/** The value of RTTARFILE::u32Magic. (Abraham Stoker) */
-#define RTTARFILE_MAGIC                 UINT32_C(0x18471108)
-/** The value of RTTARFILE::u32Magic after RTTarFileClose(). */
-#define RTTARFILE_MAGIC_DEAD            UINT32_C(0x19120420)
 /** RTTESTINT::u32Magic value. (Daniel Kehlmann) */
 #define RTTESTINT_MAGIC                 UINT32_C(0x19750113)
 /** RTTHREADCTXHOOKINT::u32Magic value. (Dennis MacAlistair Ritchie) */
