@@ -3,44 +3,57 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-#ifndef ___iprt_asm_amd64_x86_h
-#define ___iprt_asm_amd64_x86_h
+#ifndef IPRT_INCLUDED_asm_amd64_x86_h
+#define IPRT_INCLUDED_asm_amd64_x86_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/types.h>
 #include <iprt/assert.h>
+#include <iprt/x86-helpers.h>
 #if !defined(RT_ARCH_AMD64) && !defined(RT_ARCH_X86)
 # error "Not on AMD64 or x86"
 #endif
 
 #if defined(_MSC_VER) && RT_INLINE_ASM_USES_INTRIN
-# pragma warning(push)
-# pragma warning(disable:4668) /* Several incorrect __cplusplus uses. */
-# pragma warning(disable:4255) /* Incorrect __slwpcb prototype. */
-# include <intrin.h>
-# pragma warning(pop)
-   /* Emit the intrinsics at all optimization levels. */
+/* Emit the intrinsics at all optimization levels. */
+# include <iprt/sanitized/intrin.h>
 # pragma intrinsic(_ReadWriteBarrier)
 # pragma intrinsic(__cpuid)
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2010 /*?*/
+#  pragma intrinsic(__cpuidex)
+# endif
 # pragma intrinsic(_enable)
 # pragma intrinsic(_disable)
 # pragma intrinsic(__rdtsc)
@@ -73,14 +86,26 @@
 #  pragma intrinsic(__readcr8)
 #  pragma intrinsic(__writecr8)
 # endif
-# if RT_INLINE_ASM_USES_INTRIN >= 14
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2005
 #  pragma intrinsic(__halt)
 # endif
-# if RT_INLINE_ASM_USES_INTRIN >= 15
-#  pragma intrinsic(__readeflags)
-#  pragma intrinsic(__writeeflags)
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
+/*#  pragma intrinsic(__readeflags)  - buggy intrinsics in VC++ 2010, reordering/optimizers issues
+#  pragma intrinsic(__writeeflags) */
 #  pragma intrinsic(__rdtscp)
 # endif
+# if defined(RT_ARCH_AMD64) && RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2015 /*?*/
+#  pragma intrinsic(_readfsbase_u64)
+#  pragma intrinsic(_readgsbase_u64)
+#  pragma intrinsic(_writefsbase_u64)
+#  pragma intrinsic(_writegsbase_u64)
+# endif
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+#  pragma intrinsic(__lidt)
+#  pragma intrinsic(__sidt)
+#  pragma intrinsic(_lgdt)
+#  pragma intrinsic(_sgdt)
+# endif
 #endif
 
 
@@ -183,12 +208,14 @@ typedef RTGDTRALIGNED RT_FAR *PRGDTRALIGNED;
  * Gets the content of the IDTR CPU register.
  * @param   pIdtr   Where to store the IDTR contents.
  */
-#if RT_INLINE_ASM_EXTERNAL
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2013
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMGetIDTR(PRTIDTR pIdtr);
 #else
 DECLINLINE(void) ASMGetIDTR(PRTIDTR pIdtr)
 {
-# if RT_INLINE_ASM_GNU_STYLE
+# if  RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+    __sidt(pIdtr);
+# elif RT_INLINE_ASM_GNU_STYLE
     __asm__ __volatile__("sidt %0" : "=m" (*pIdtr));
 # else
     __asm
@@ -210,13 +237,15 @@ DECLINLINE(void) ASMGetIDTR(PRTIDTR pIdtr)
  * Gets the content of the IDTR.LIMIT CPU register.
  * @returns IDTR limit.
  */
-#if RT_INLINE_ASM_EXTERNAL
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2013
 RT_ASM_DECL_PRAGMA_WATCOM(uint16_t) ASMGetIdtrLimit(void);
 #else
 DECLINLINE(uint16_t) ASMGetIdtrLimit(void)
 {
     RTIDTRALIGNED TmpIdtr;
-# if RT_INLINE_ASM_GNU_STYLE
+# if  RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+    __sidt(&TmpIdtr);
+# elif RT_INLINE_ASM_GNU_STYLE
     __asm__ __volatile__("sidt %0" : "=m" (TmpIdtr.s.Idtr));
 # else
     __asm
@@ -233,12 +262,14 @@ DECLINLINE(uint16_t) ASMGetIdtrLimit(void)
  * Sets the content of the IDTR CPU register.
  * @param   pIdtr   Where to load the IDTR contents from
  */
-#if RT_INLINE_ASM_EXTERNAL
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2013
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMSetIDTR(const RTIDTR RT_FAR *pIdtr);
 #else
 DECLINLINE(void) ASMSetIDTR(const RTIDTR RT_FAR *pIdtr)
 {
-# if RT_INLINE_ASM_GNU_STYLE
+# if  RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+    __lidt((void *)pIdtr);
+# elif RT_INLINE_ASM_GNU_STYLE
     __asm__ __volatile__("lidt %0" : : "m" (*pIdtr));
 # else
     __asm
@@ -260,12 +291,14 @@ DECLINLINE(void) ASMSetIDTR(const RTIDTR RT_FAR *pIdtr)
  * Gets the content of the GDTR CPU register.
  * @param   pGdtr   Where to store the GDTR contents.
  */
-#if RT_INLINE_ASM_EXTERNAL
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2013
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMGetGDTR(PRTGDTR pGdtr);
 #else
 DECLINLINE(void) ASMGetGDTR(PRTGDTR pGdtr)
 {
-# if RT_INLINE_ASM_GNU_STYLE
+# if  RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+    _sgdt(pGdtr);
+# elif RT_INLINE_ASM_GNU_STYLE
     __asm__ __volatile__("sgdt %0" : "=m" (*pGdtr));
 # else
     __asm
@@ -287,12 +320,14 @@ DECLINLINE(void) ASMGetGDTR(PRTGDTR pGdtr)
  * Sets the content of the GDTR CPU register.
  * @param   pGdtr   Where to load the GDTR contents from
  */
-#if RT_INLINE_ASM_EXTERNAL
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2013
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMSetGDTR(const RTGDTR RT_FAR *pGdtr);
 #else
 DECLINLINE(void) ASMSetGDTR(const RTGDTR RT_FAR *pGdtr)
 {
-# if RT_INLINE_ASM_GNU_STYLE
+# if  RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2013
+    _lgdt((void *)pGdtr);
+# elif RT_INLINE_ASM_GNU_STYLE
     __asm__ __volatile__("lgdt %0" : : "m" (*pGdtr));
 # else
     __asm
@@ -406,6 +441,46 @@ DECLINLINE(RTSEL) ASMGetFS(void)
 }
 # endif
 
+#ifdef RT_ARCH_AMD64
+
+/**
+ * Get the FS base register.
+ * @returns FS base address.
+ */
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2015 /*?*/
+DECLASM(uint64_t) ASMGetFSBase(void);
+#else
+DECLINLINE(uint64_t) ASMGetFSBase(void)
+{
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2015
+    return (uint64_t)_readfsbase_u64();
+# elif RT_INLINE_ASM_GNU_STYLE
+    uint64_t uFSBase;
+    __asm__ __volatile__("rdfsbase %0\n\t" : "=r" (uFSBase));
+    return uFSBase;
+# endif
+}
+# endif
+
+
+/**
+ * Set the FS base register.
+ * @param   uNewBase    The new base value.
+ */
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2015 /*?*/
+DECLASM(void) ASMSetFSBase(uint64_t uNewBase);
+#else
+DECLINLINE(void) ASMSetFSBase(uint64_t uNewBase)
+{
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2015
+    _writefsbase_u64(uNewBase);
+# elif RT_INLINE_ASM_GNU_STYLE
+    __asm__ __volatile__("wrfsbase %0\n\t" : : "r" (uNewBase));
+# endif
+}
+# endif
+
+#endif /* RT_ARCH_AMD64 */
 
 /**
  * Get the GS register.
@@ -430,6 +505,47 @@ DECLINLINE(RTSEL) ASMGetGS(void)
 }
 #endif
 
+#ifdef RT_ARCH_AMD64
+
+/**
+ * Get the GS base register.
+ * @returns GS base address.
+ */
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2015 /*?*/
+DECLASM(uint64_t) ASMGetGSBase(void);
+#else
+DECLINLINE(uint64_t) ASMGetGSBase(void)
+{
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2015
+    return (uint64_t)_readgsbase_u64();
+# elif RT_INLINE_ASM_GNU_STYLE
+    uint64_t uGSBase;
+    __asm__ __volatile__("rdgsbase %0\n\t" : "=r" (uGSBase));
+    return uGSBase;
+# endif
+}
+# endif
+
+
+/**
+ * Set the GS base register.
+ * @param   uNewBase    The new base value.
+ */
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2015 /*?*/
+DECLASM(void) ASMSetGSBase(uint64_t uNewBase);
+#else
+DECLINLINE(void) ASMSetGSBase(uint64_t uNewBase)
+{
+# if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2015
+    _writegsbase_u64(uNewBase);
+# elif RT_INLINE_ASM_GNU_STYLE
+    __asm__ __volatile__("wrgsbase %0\n\t" : : "r" (uNewBase));
+# endif
+}
+# endif
+
+#endif /* RT_ARCH_AMD64 */
+
 
 /**
  * Get the SS register.
@@ -548,7 +664,7 @@ DECLINLINE(uint32_t) ASMGetSegAttr(uint32_t uSel)
  * Get the [RE]FLAGS register.
  * @returns [RE]FLAGS.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL /*&& RT_INLINE_ASM_USES_INTRIN < 15 - buggy intrinsics in VC++ 2010, reordering/optimizers issues. */
 RT_ASM_DECL_PRAGMA_WATCOM(RTCCUINTREG) ASMGetFlags(void);
 #else
 DECLINLINE(RTCCUINTREG) ASMGetFlags(void)
@@ -564,7 +680,7 @@ DECLINLINE(RTCCUINTREG) ASMGetFlags(void)
                          "popl  %0\n\t"
                          : "=r" (uFlags));
 #  endif
-# elif RT_INLINE_ASM_USES_INTRIN >= 15
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     uFlags = __readeflags();
 # else
     __asm
@@ -587,7 +703,7 @@ DECLINLINE(RTCCUINTREG) ASMGetFlags(void)
  * Set the [RE]FLAGS register.
  * @param   uFlags      The new [RE]FLAGS value.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL /*&& RT_INLINE_ASM_USES_INTRIN < 15 - see __readeflags() above. */
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMSetFlags(RTCCUINTREG uFlags);
 #else
 DECLINLINE(void) ASMSetFlags(RTCCUINTREG uFlags)
@@ -602,7 +718,7 @@ DECLINLINE(void) ASMSetFlags(RTCCUINTREG uFlags)
                          "popfl\n\t"
                          : : "g" (uFlags));
 #  endif
-# elif RT_INLINE_ASM_USES_INTRIN >= 15
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     __writeeflags(uFlags);
 # else
     __asm
@@ -626,7 +742,7 @@ DECLINLINE(void) ASMSetFlags(RTCCUINTREG uFlags)
  * @param   fAndEfl     Flags to keep (applied first).
  * @param   fOrEfl      Flags to be set.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL /*&& RT_INLINE_ASM_USES_INTRIN < 15 - buggy intrinsics in VC++ 2010, reordering/optimizers issues. */
 RT_ASM_DECL_PRAGMA_WATCOM(RTCCUINTREG) ASMChangeFlags(RTCCUINTREG fAndEfl, RTCCUINTREG fOrEfl);
 #else
 DECLINLINE(RTCCUINTREG) ASMChangeFlags(RTCCUINTREG fAndEfl, RTCCUINTREG fOrEfl)
@@ -654,7 +770,7 @@ DECLINLINE(RTCCUINTREG) ASMChangeFlags(RTCCUINTREG fAndEfl, RTCCUINTREG fOrEfl)
                          : "rn" (fAndEfl),
                            "rn" (fOrEfl) );
 #  endif
-# elif RT_INLINE_ASM_USES_INTRIN >= 15
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     fOldEfl = __readeflags();
     __writeeflags((fOldEfl & fAndEfl) | fOrEfl);
 # else
@@ -693,7 +809,7 @@ DECLINLINE(RTCCUINTREG) ASMChangeFlags(RTCCUINTREG fAndEfl, RTCCUINTREG fOrEfl)
  * @returns Original value.
  * @param   fOrEfl      The flags to be set (ORed in).
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL /*&& RT_INLINE_ASM_USES_INTRIN < 15 - buggy intrinsics in VC++ 2010, reordering/optimizers issues. */
 RT_ASM_DECL_PRAGMA_WATCOM(RTCCUINTREG) ASMAddFlags(RTCCUINTREG fOrEfl);
 #else
 DECLINLINE(RTCCUINTREG) ASMAddFlags(RTCCUINTREG fOrEfl)
@@ -715,7 +831,7 @@ DECLINLINE(RTCCUINTREG) ASMAddFlags(RTCCUINTREG fOrEfl)
                          : "=&r" (fOldEfl)
                          : "rn" (fOrEfl) );
 #  endif
-# elif RT_INLINE_ASM_USES_INTRIN >= 15
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     fOldEfl = __readeflags();
     __writeeflags(fOldEfl | fOrEfl);
 # else
@@ -748,7 +864,7 @@ DECLINLINE(RTCCUINTREG) ASMAddFlags(RTCCUINTREG fOrEfl)
  * @returns Original value.
  * @param   fAndEfl      The flags to keep.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL /*&& RT_INLINE_ASM_USES_INTRIN < 15 - buggy intrinsics in VC++ 2010, reordering/optimizers issues. */
 RT_ASM_DECL_PRAGMA_WATCOM(RTCCUINTREG) ASMClearFlags(RTCCUINTREG fAndEfl);
 #else
 DECLINLINE(RTCCUINTREG) ASMClearFlags(RTCCUINTREG fAndEfl)
@@ -770,7 +886,7 @@ DECLINLINE(RTCCUINTREG) ASMClearFlags(RTCCUINTREG fAndEfl)
                          : "=&r" (fOldEfl)
                          : "rn" (fAndEfl) );
 #  endif
-# elif RT_INLINE_ASM_USES_INTRIN >= 15
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     fOldEfl = __readeflags();
     __writeeflags(fOldEfl & fAndEfl);
 # else
@@ -835,7 +951,7 @@ DECLINLINE(uint64_t) ASMReadTSC(void)
  * @returns TSC.
  * @param   puAux   Where to store the AUX value.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 15
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2008
 RT_ASM_DECL_PRAGMA_WATCOM(uint64_t) ASMReadTscWithAux(uint32_t RT_FAR *puAux);
 #else
 DECLINLINE(uint64_t) ASMReadTscWithAux(uint32_t RT_FAR *puAux)
@@ -846,7 +962,7 @@ DECLINLINE(uint64_t) ASMReadTscWithAux(uint32_t RT_FAR *puAux)
     /*__asm__ __volatile__("rdtscp\n\t" : "=a" (u.s.Lo), "=d" (u.s.Hi), "=c" (*puAux)); */
     __asm__ __volatile__(".byte 0x0f,0x01,0xf9\n\t" : "=a" (u.s.Lo), "=d" (u.s.Hi), "=c" (*puAux));
 # else
-#  if RT_INLINE_ASM_USES_INTRIN >= 15
+#  if RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2008
     u.u = __rdtscp(puAux);
 #  else
     __asm
@@ -1382,18 +1498,80 @@ DECLINLINE(uint8_t) ASMGetApicId(void)
 
 
 /**
- * Tests if it a genuine Intel CPU based on the ASMCpuId(0) output.
+ * Gets the APIC ID of the current CPU using leaf 0xb.
  *
- * @returns true/false.
- * @param   uEBX    EBX return from ASMCpuId(0)
- * @param   uECX    ECX return from ASMCpuId(0)
- * @param   uEDX    EDX return from ASMCpuId(0)
+ * @returns the APIC ID.
  */
-DECLINLINE(bool) ASMIsIntelCpuEx(uint32_t uEBX, uint32_t uECX, uint32_t uEDX)
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2010 /*?*/
+RT_ASM_DECL_PRAGMA_WATCOM(uint32_t) ASMGetApicIdExt0B(void);
+#else
+DECLINLINE(uint32_t) ASMGetApicIdExt0B(void)
 {
-    return uEBX == UINT32_C(0x756e6547)
-        && uECX == UINT32_C(0x6c65746e)
-        && uEDX == UINT32_C(0x49656e69);
+# if RT_INLINE_ASM_GNU_STYLE
+    RTCCUINTREG xDX;
+#  ifdef RT_ARCH_AMD64
+    RTCCUINTREG uSpillEax, uSpillEcx;
+    __asm__ __volatile__ ("cpuid"
+                          : "=a" (uSpillEax),
+                            "=c" (uSpillEcx),
+                            "=d" (xDX)
+                          : "0" (0xb),
+                            "1" (0)
+                          : "rbx");
+#  elif (defined(PIC) || defined(__PIC__)) && defined(__i386__)
+    RTCCUINTREG uSpillEax, uSpillEcx, uSpillEbx;
+    __asm__ __volatile__ ("mov   %%ebx,%2\n\t"
+                          "cpuid\n\t"
+                          "xchgl %%ebx,%2\n\t"
+                          : "=a" (uSpillEax),
+                            "=c" (uSpillEcx),
+                            "=rm" (uSpillEbx),
+                            "=d" (xDX)
+                          : "0" (0xb),
+                            "1" (0));
+#  else
+    RTCCUINTREG uSpillEax, uSpillEcx;
+    __asm__ __volatile__ ("cpuid"
+                          : "=a" (uSpillEax),
+                            "=c" (uSpillEcx),
+                            "=d" (xDX)
+                          : "0" (0xb),
+                            "1" (0)
+                          : "ebx");
+#  endif
+    return (uint32_t)xDX;
+
+# elif RT_INLINE_ASM_USES_INTRIN >= RT_MSC_VER_VS2010 /*?*/
+
+    int aInfo[4];
+    __cpuidex(aInfo, 0xb, 0);
+    return aInfo[3];
+
+# else
+    RTCCUINTREG xDX;
+    __asm
+    {
+        push    ebx
+        mov     eax, 0xb
+        xor     ecx, ecx
+        cpuid
+        mov     [xDX], edx
+        pop     ebx
+    }
+    return (uint32_t)xDX;
+# endif
+}
+#endif
+
+
+/**
+ * Gets the APIC ID of the current CPU using leaf 8000001E.
+ *
+ * @returns the APIC ID.
+ */
+DECLINLINE(uint32_t) ASMGetApicIdExt8000001E(void)
+{
+    return ASMCpuId_EAX(0x8000001e);
 }
 
 
@@ -1407,23 +1585,7 @@ DECLINLINE(bool) ASMIsIntelCpu(void)
 {
     uint32_t uEAX, uEBX, uECX, uEDX;
     ASMCpuId(0, &uEAX, &uEBX, &uECX, &uEDX);
-    return ASMIsIntelCpuEx(uEBX, uECX, uEDX);
-}
-
-
-/**
- * Tests if it an authentic AMD CPU based on the ASMCpuId(0) output.
- *
- * @returns true/false.
- * @param   uEBX    EBX return from ASMCpuId(0)
- * @param   uECX    ECX return from ASMCpuId(0)
- * @param   uEDX    EDX return from ASMCpuId(0)
- */
-DECLINLINE(bool) ASMIsAmdCpuEx(uint32_t uEBX, uint32_t uECX, uint32_t uEDX)
-{
-    return uEBX == UINT32_C(0x68747541)
-        && uECX == UINT32_C(0x444d4163)
-        && uEDX == UINT32_C(0x69746e65);
+    return RTX86IsIntelCpu(uEBX, uECX, uEDX);
 }
 
 
@@ -1437,23 +1599,7 @@ DECLINLINE(bool) ASMIsAmdCpu(void)
 {
     uint32_t uEAX, uEBX, uECX, uEDX;
     ASMCpuId(0, &uEAX, &uEBX, &uECX, &uEDX);
-    return ASMIsAmdCpuEx(uEBX, uECX, uEDX);
-}
-
-
-/**
- * Tests if it a centaur hauling VIA CPU based on the ASMCpuId(0) output.
- *
- * @returns true/false.
- * @param   uEBX    EBX return from ASMCpuId(0).
- * @param   uECX    ECX return from ASMCpuId(0).
- * @param   uEDX    EDX return from ASMCpuId(0).
- */
-DECLINLINE(bool) ASMIsViaCentaurCpuEx(uint32_t uEBX, uint32_t uECX, uint32_t uEDX)
-{
-    return uEBX == UINT32_C(0x746e6543)
-        && uECX == UINT32_C(0x736c7561)
-        && uEDX == UINT32_C(0x48727561);
+    return RTX86IsAmdCpu(uEBX, uECX, uEDX);
 }
 
 
@@ -1467,129 +1613,35 @@ DECLINLINE(bool) ASMIsViaCentaurCpu(void)
 {
     uint32_t uEAX, uEBX, uECX, uEDX;
     ASMCpuId(0, &uEAX, &uEBX, &uECX, &uEDX);
-    return ASMIsViaCentaurCpuEx(uEBX, uECX, uEDX);
+    return RTX86IsViaCentaurCpu(uEBX, uECX, uEDX);
 }
 
 
 /**
- * Checks whether ASMCpuId_EAX(0x00000000) indicates a valid range.
- *
+ * Tests if this is a Shanghai CPU.
  *
  * @returns true/false.
- * @param   uEAX    The EAX value of CPUID leaf 0x00000000.
- *
- * @note    This only succeeds if there are at least two leaves in the range.
- * @remarks The upper range limit is just some half reasonable value we've
- *          picked out of thin air.
+ * @remarks ASSUMES that cpuid is supported by the CPU.
  */
-DECLINLINE(bool) ASMIsValidStdRange(uint32_t uEAX)
+DECLINLINE(bool) ASMIsShanghaiCpu(void)
 {
-    return uEAX >= UINT32_C(0x00000001) && uEAX <= UINT32_C(0x000fffff);
+    uint32_t uEAX, uEBX, uECX, uEDX;
+    ASMCpuId(0, &uEAX, &uEBX, &uECX, &uEDX);
+    return RTX86IsShanghaiCpu(uEBX, uECX, uEDX);
 }
 
 
 /**
- * Checks whether ASMCpuId_EAX(0x80000000) indicates a valid range.
- *
- * This only succeeds if there are at least two leaves in the range.
+ * Tests if this is a genuine Hygon CPU.
  *
  * @returns true/false.
- * @param   uEAX    The EAX value of CPUID leaf 0x80000000.
- *
- * @note    This only succeeds if there are at least two leaves in the range.
- * @remarks The upper range limit is just some half reasonable value we've
- *          picked out of thin air.
+ * @remarks ASSUMES that cpuid is supported by the CPU.
  */
-DECLINLINE(bool) ASMIsValidExtRange(uint32_t uEAX)
+DECLINLINE(bool) ASMIsHygonCpu(void)
 {
-    return uEAX >= UINT32_C(0x80000001) && uEAX <= UINT32_C(0x800fffff);
-}
-
-
-/**
- * Checks whether ASMCpuId_EAX(0x40000000) indicates a valid range.
- *
- * This only succeeds if there are at least two leaves in the range.
- *
- * @returns true/false.
- * @param   uEAX    The EAX value of CPUID leaf 0x40000000.
- *
- * @note    Unlike ASMIsValidStdRange() and ASMIsValidExtRange(), a single leaf
- *          is okay here.  So, you always need to check the range.
- * @remarks The upper range limit is take from the intel docs.
- */
-DECLINLINE(bool) ASMIsValidHypervisorRange(uint32_t uEAX)
-{
-    return uEAX >= UINT32_C(0x40000000) && uEAX <= UINT32_C(0x4fffffff);
-}
-
-
-/**
- * Extracts the CPU family from ASMCpuId(1) or ASMCpuId(0x80000001)
- *
- * @returns Family.
- * @param   uEAX    EAX return from ASMCpuId(1) or ASMCpuId(0x80000001).
- */
-DECLINLINE(uint32_t) ASMGetCpuFamily(uint32_t uEAX)
-{
-    return ((uEAX >> 8) & 0xf) == 0xf
-         ? ((uEAX >> 20) & 0x7f) + 0xf
-         : ((uEAX >> 8) & 0xf);
-}
-
-
-/**
- * Extracts the CPU model from ASMCpuId(1) or ASMCpuId(0x80000001), Intel variant.
- *
- * @returns Model.
- * @param   uEAX    EAX from ASMCpuId(1) or ASMCpuId(0x80000001).
- */
-DECLINLINE(uint32_t) ASMGetCpuModelIntel(uint32_t uEAX)
-{
-    return ((uEAX >> 8) & 0xf) == 0xf || (((uEAX >> 8) & 0xf) == 0x6) /* family! */
-         ? ((uEAX >> 4) & 0xf) | ((uEAX >> 12) & 0xf0)
-         : ((uEAX >> 4) & 0xf);
-}
-
-
-/**
- * Extracts the CPU model from ASMCpuId(1) or ASMCpuId(0x80000001), AMD variant.
- *
- * @returns Model.
- * @param   uEAX    EAX from ASMCpuId(1) or ASMCpuId(0x80000001).
- */
-DECLINLINE(uint32_t) ASMGetCpuModelAMD(uint32_t uEAX)
-{
-    return ((uEAX >> 8) & 0xf) == 0xf
-         ? ((uEAX >> 4) & 0xf) | ((uEAX >> 12) & 0xf0)
-         : ((uEAX >> 4) & 0xf);
-}
-
-
-/**
- * Extracts the CPU model from ASMCpuId(1) or ASMCpuId(0x80000001)
- *
- * @returns Model.
- * @param   uEAX    EAX from ASMCpuId(1) or ASMCpuId(0x80000001).
- * @param   fIntel  Whether it's an intel CPU. Use ASMIsIntelCpuEx() or ASMIsIntelCpu().
- */
-DECLINLINE(uint32_t) ASMGetCpuModel(uint32_t uEAX, bool fIntel)
-{
-    return ((uEAX >> 8) & 0xf) == 0xf || (((uEAX >> 8) & 0xf) == 0x6 && fIntel) /* family! */
-         ? ((uEAX >> 4) & 0xf) | ((uEAX >> 12) & 0xf0)
-         : ((uEAX >> 4) & 0xf);
-}
-
-
-/**
- * Extracts the CPU stepping from ASMCpuId(1) or ASMCpuId(0x80000001)
- *
- * @returns Model.
- * @param   uEAX    EAX from ASMCpuId(1) or ASMCpuId(0x80000001).
- */
-DECLINLINE(uint32_t) ASMGetCpuStepping(uint32_t uEAX)
-{
-    return uEAX & 0xf;
+    uint32_t uEAX, uEBX, uECX, uEDX;
+    ASMCpuId(0, &uEAX, &uEBX, &uECX, &uEDX);
+    return RTX86IsHygonCpu(uEBX, uECX, uEDX);
 }
 
 
@@ -2080,7 +2132,7 @@ DECLINLINE(bool) ASMIntAreEnabled(void)
 /**
  * Halts the CPU until interrupted.
  */
-#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < 14
+#if RT_INLINE_ASM_EXTERNAL && RT_INLINE_ASM_USES_INTRIN < RT_MSC_VER_VS2005
 RT_ASM_DECL_PRAGMA_WATCOM(void) ASMHalt(void);
 #else
 DECLINLINE(void) ASMHalt(void)
@@ -2137,12 +2189,11 @@ DECLINLINE(uint64_t) ASMRdMsr(uint32_t uRegister)
 /**
  * Writes a machine specific register.
  *
- * @returns Register content.
  * @param   uRegister   Register to write to.
  * @param   u64Val      Value to write.
  */
 #if RT_INLINE_ASM_EXTERNAL && !RT_INLINE_ASM_USES_INTRIN
-RT_ASM_DECL_PRAGMA_WATCOM(void) ASMWrMsr(uint32_t uRegister, uint64_t u64Val);
+RT_ASM_DECL_PRAGMA_WATCOM_386(void) ASMWrMsr(uint32_t uRegister, uint64_t u64Val);
 #else
 DECLINLINE(void) ASMWrMsr(uint32_t uRegister, uint64_t u64Val)
 {
@@ -2179,7 +2230,7 @@ DECLINLINE(void) ASMWrMsr(uint32_t uRegister, uint64_t u64Val)
  * @param   uXDI        RDI/EDI value.
  */
 #if RT_INLINE_ASM_EXTERNAL
-RT_ASM_DECL_PRAGMA_WATCOM(uint64_t) ASMRdMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI);
+RT_ASM_DECL_PRAGMA_WATCOM_386(uint64_t) ASMRdMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI);
 #else
 DECLINLINE(uint64_t) ASMRdMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI)
 {
@@ -2211,13 +2262,12 @@ DECLINLINE(uint64_t) ASMRdMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI)
 /**
  * Writes a machine specific register, extended version (for AMD).
  *
- * @returns Register content.
  * @param   uRegister   Register to write to.
  * @param   uXDI        RDI/EDI value.
  * @param   u64Val      Value to write.
  */
 #if RT_INLINE_ASM_EXTERNAL
-RT_ASM_DECL_PRAGMA_WATCOM(void) ASMWrMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI, uint64_t u64Val);
+RT_ASM_DECL_PRAGMA_WATCOM_386(void) ASMWrMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI, uint64_t u64Val);
 #else
 DECLINLINE(void) ASMWrMsrEx(uint32_t uRegister, RTCCUINTXREG uXDI, uint64_t u64Val)
 {
@@ -3381,15 +3431,15 @@ DECLINLINE(void) ASMSetAC(void)
  */
 #if defined(__WATCOMC__) && ARCH_BITS == 16
 # define IPRT_ASM_AMD64_X86_WATCOM_16_INSTANTIATE
-# undef ___iprt_asm_amd64_x86_watcom_16_h
+# undef IPRT_INCLUDED_asm_amd64_x86_watcom_16_h
 # include "asm-amd64-x86-watcom-16.h"
 #elif defined(__WATCOMC__) && ARCH_BITS == 32
 # define IPRT_ASM_AMD64_X86_WATCOM_32_INSTANTIATE
-# undef ___iprt_asm_amd64_x86_watcom_32_h
+# undef IPRT_INCLUDED_asm_amd64_x86_watcom_32_h
 # include "asm-amd64-x86-watcom-32.h"
 #endif
 
 
 /** @} */
-#endif
+#endif /* !IPRT_INCLUDED_asm_amd64_x86_h */
 
