@@ -1,4 +1,4 @@
-/* $Id: SUPDrv-linux.c 155244 2023-01-17 14:15:46Z bird $ */
+/* $Id: SUPDrv-linux.c 162704 2024-04-09 12:24:51Z aeichner $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Linux specifics.
  */
@@ -54,6 +54,7 @@
 #include <VBox/err.h>
 #include <iprt/mem.h>
 #include <VBox/log.h>
+#include <VBox/VBoxLnxModInline.h>
 #include <iprt/mp.h>
 
 /** @todo figure out the exact version number */
@@ -307,12 +308,12 @@ DECLINLINE(RTGID) vboxdrvLinuxKernGid(void)
 
 
 #ifdef VBOX_WITH_HARDENING
-/** Get the effective UID within the current user namespace. */
-DECLINLINE(RTUID) vboxdrvLinuxEuidInNs(void)
+/** Get the effective UID of the current process. */
+DECLINLINE(RTUID) vboxdrvLinuxEuid(void)
 {
 # if RTLNX_VER_MIN(2,6,29)
 #  if RTLNX_VER_MIN(3,5,0)
-    return from_kuid(current_user_ns(), current->cred->euid);
+    return __kuid_val(current->cred->euid);
 #  else
     return current->cred->euid;
 #  endif
@@ -332,6 +333,10 @@ static int __init VBoxDrvLinuxInit(void)
 {
     int       rc;
 
+    /* Check if modue loading was disabled. */
+    if (!vbox_mod_should_load())
+        return -EINVAL;
+
 #if RTLNX_VER_MIN(5,0,0)
     spin_lock_init(&g_supdrvLinuxWrapperModuleSpinlock);
     RTListInit(&g_supdrvLinuxWrapperModuleList);
@@ -487,9 +492,9 @@ static int vboxdrvLinuxCreateCommon(struct inode *pInode, struct file *pFilp, bo
      * Only root is allowed to access the unrestricted device, enforce it!
      */
     if (   fUnrestricted
-        && vboxdrvLinuxEuidInNs() != 0 /* root */ )
+        && vboxdrvLinuxEuid() != 0 /* root */ )
     {
-        Log(("VBoxDrvLinuxCreate: euid=%d, expected 0 (root)\n", vboxdrvLinuxEuidInNs()));
+        Log(("VBoxDrvLinuxCreate: euid=%d, expected 0 (root)\n", vboxdrvLinuxEuid()));
         return -EPERM;
     }
 #endif /* VBOX_WITH_HARDENING */
@@ -1683,12 +1688,25 @@ SUPR0_EXPORT_SYMBOL(SUPR0PrintfV);
 SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
 {
     uint32_t fFlags = 0;
+    /* Note! bird 2023-10-20: Apparently, with CONFIG_PAX_KERNEXEC these days,
+             not only is the regular GDT read-only, but the one returned by
+             get_current_gdt_rw() is also read-only despite the name.
+
+             We don't know exactly when this started, or if it was always like
+             this, but getting hold of the relevant patches isn't all that
+             straight forward any longer it seems (which is weird for linux
+             patches), so, we've just enabled slow-mode for all PAX_KERNEXEC
+             kernels regardless of kernel version.
+
+             Looking at grsecurity patch for 4.9.9, it looks like the writable
+             GDT stuff never worked with PaX/grsec.
+             */
 #ifdef CONFIG_PAX_KERNEXEC
     fFlags |= SUPKERNELFEATURES_GDT_READ_ONLY;
-#endif
-#if RTLNX_VER_MIN(4,12,0)
+#elif RTLNX_VER_MIN(4,12,0)
     fFlags |= SUPKERNELFEATURES_GDT_NEED_WRITABLE;
 #endif
+
 #if defined(VBOX_STRICT) || defined(VBOX_WITH_EFLAGS_AC_SET_IN_VBOXDRV)
     fFlags |= SUPKERNELFEATURES_SMAP;
 #elif defined(CONFIG_X86_SMAP)
@@ -1759,7 +1777,7 @@ SUPR0_EXPORT_SYMBOL(SUPR0FpuEnd);
 
 int VBOXCALL    supdrvOSGetCurrentGdtRw(RTHCUINTPTR *pGdtRw)
 {
-#if RTLNX_VER_MIN(4,12,0)
+#if RTLNX_VER_MIN(4,12,0) && !defined(CONFIG_PAX_KERNEXEC)
     *pGdtRw = (RTHCUINTPTR)get_current_gdt_rw();
     return VINF_SUCCESS;
 #else
